@@ -7,24 +7,29 @@
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #    GNU General Public License for more details. (see file LICENSE)
-#    see also <http://www.gnu.org/licenses/>.
+#    see also <http://www.gnu.org/licenses/>. 
+#   Dr Didier G.Leibovici CC 2001-20010-2012
 
 "howtoPTAk" <-
 function()
 {
-cat("          Copyright 2000, 2007, 2010 Didier Leibovici" , "\n",
-    "            to see a full description of the licence ","\n",
-    "            open the file LICENSE given in the zip or gz or in","\n",
-    paste(.libPaths(),"/PTAk",sep="")," or contact me at c3s2i@free.fr","\n")
-cat("         see the examples on http://c3s2i.free.fr","\n")
-file.show(paste(.libPaths(), "/PTAk/DESCRIPTION", sep = ""))    
+	print(packageDescription("PTAk"))
+		cat( "\n")
+		cat("********************", "\n",
+	"          Copyright GPL >=2 2000, 2007, 2010, 2012 Didier Leibovici" , "\n",
+    "            see the citation file","\n",
+    "            and for a good introduction ","\n",
+    "           Leibovici, D.G. (2010) JSS:34(10), www.jstatsoft.org/v34/i10/","\n",
+    "          contact me at c3s2i@free.fr","\n")
+cat("         see some examples on http://c3s2i.free.fr","\n")
+   
 }
 "CANDPARA" <-
 function (X, dim = 3, test = 1e-08, Maxiter = 1000, smoothing = FALSE,
     smoo = list(NA), verbose = getOption("verbose"), file = NULL, 
     modesnam = NULL, addedcomment = "")
 {
-    datanam <- substitute(X)
+   datanam <- substitute(X)
     sym <- NULL
     if (is.list(X)) {
         if (is.list(X$met)) 
@@ -55,7 +60,7 @@ function (X, dim = 3, test = 1e-08, Maxiter = 1000, smoothing = FALSE,
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:diX, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -95,6 +100,9 @@ function (X, dim = 3, test = 1e-08, Maxiter = 1000, smoothing = FALSE,
         stop(paste("--- X must be an array  ! ---"))
     }
     ord <- length(dim(X))
+    if (ord <=2) {
+        stop(paste("--- X must be an array  of k > 2 entries! ---"))
+    }
     if (is.null(modesnam)) {
         modesnam <- paste(rep("mo", ord), 1:ord)
     }
@@ -267,7 +275,7 @@ function (X, dim = c(2, 2, 2, 3), test = 1e-12, Maxiter = 400,
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:diX, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -305,6 +313,9 @@ function (X, dim = c(2, 2, 2, 3), test = 1e-12, Maxiter = 400,
     }
     solutions <- NULL
     ord <- length(dim(X))
+    if (ord <=2) {
+        stop(paste("--- X must be an array  of k > 2 entries! ---"))
+    }
     if (smoothing) {
         if (length(smoo) < ord) 
             smoo <- rep(list(smoo[[1]]), ord)
@@ -459,7 +470,6 @@ function (solu)
 function (X, z, Xwiz = NULL, zwiX = NULL, rezwiX = FALSE, usetensor = TRUE)
 {
     if (usetensor) {
-   	 library(tensor)
         if (is.null(zwiX)) {
             if (is.vector(z))
                 zwiX <- 1
@@ -796,7 +806,7 @@ function (solb, sola = NULL, numass = NULL, verbose = getOption("verbose"),
                 pctota > testvar]), "%", "\n")
     }
     if (!is.null(file)) {
-        cat("               ------Percent Rebuilt from Slected ----",
+        cat("               ------Percent Rebuilt from Selected ----",
             sum(pctota[!substr(sola[[k]]$vsnam, 1, 1) == "*" &
                 pctota > testvar]), "%", "\n", file = file, append = TRUE)
         if (verbose) {
@@ -822,7 +832,7 @@ function (solb, sola = NULL, numass = NULL, verbose = getOption("verbose"),
         nostar <- !substr(sola[[k]]$vsnam, 1, 1) == "*"
         cat("               data= ", deparse(sola[[k]]$datanam),
             " ", di, "\n")
-        cat("   ", sola[[k]]$addedcomment, "\n")
+        if(sola[[k]]$addedcomment!="")cat("   ", sola[[k]]$addedcomment, "\n")
         cat("                ------Percent Rebuilt----", sum(pctota[nostar]),
             "%", "\n")
         summ <- matrix(cbind(1:length(sola[[k]]$d), sola[[k]]$d,
@@ -842,7 +852,7 @@ function (solb, sola = NULL, numass = NULL, verbose = getOption("verbose"),
                 " PT  with var>", testvar, "% total", "\n")
         else cat(" over ", length(sola[[k]]$vsnam[nostar]), " PT ",
             "\n")
-             invisible(summ)
+       invisible(summ)
     }
     else invisible(sola)
 }
@@ -876,58 +886,68 @@ function (T, moins = NULL, asarray = TRUE, order = NULL, id = NULL)
     }
     return(tensel)
 }
-
 "summary.FCAk" <-
 function (object, testvar = 0.5, dontshow = "*",...)
 {        solution <-object
-    nostar <- (!substr(solution[[length(solution)]]$vsnam, 1, 1) == "*")
-    if (dontshow == "*")
-        show <- (!substr(solution[[length(solution)]]$vsnam, 1, 1) ==
-            "*")
-    else if (!is.null(dontshow))
-        show <- dontshow & nostar
-    else show <- TRUE
-    k <- length(solution)
-    pctotafc <- (100 * (solution[[k]]$d)^2)/(solution[[k]]$ssX[1] - 1)
-    pctota <- (100 * (solution[[k]]$d)^2)/solution[[k]]$ssX[1]
-    cat("\n", "++++ FCA- ", k, "modes++++ ", "\n")
+      #dobjectontshow NULL == "*"
+      nostar <- (!substr(object[[length(object)]]$vsnam, 1, 1) == "*")    
+      show <-nostar
+       
+    	if (is.character(dontshow)){
+    		if(dontshow=="*") dontshow="[*]"
+        	  show <- !(grepl(dontshow, object[[length(object)]]$vsnam)) 
+        	  }
+   	 	if (is.logical(dontshow))show <-!dontshow 
+    
+   
+    k <- length(object)
+    ismodel <- "E=" %in% class(object)
+    wha="complete independence"
+    if(ismodel)wha=" model(E=) "
+    pctota <- (100 * (object[[k]]$d)^2)/object[[k]]$ssX[1]
+    pctotafc <-pctota
+    if(!ismodel)pctotafc <- (100 * (object[[k]]$d)^2)/(object[[k]]$ssX[1] - 1)
+    cat("\n", "+++ FCA- ",wha," ++ ", k, "modes+++ ", "\n")
     di <- NULL
-    for (r in 1:length(solution)) di <- c(di, length(solution[[r]]$v[1,
+    for (r in 1:length(object)) di <- c(di, length(object[[r]]$v[1,
         ]))
-    cat("     ++ Contingency Table ", deparse(solution[[k]]$datanam),
+    cat("     ++ Contingency Table ", deparse(object[[k]]$datanam),
         " ", di, " ++", "\n")
-    cat("   ", solution[[k]]$addedcomment, "\n")
-    cat("                -----Total Percent Rebuilt----", sum(pctota[nostar]),
+    if(object[[k]]$addedcomment!="")cat("   ", object[[k]]$addedcomment, "\n")
+    cat("     -----Total Percent Rebuilt----", sum(pctota[nostar]),
         "%", "\n")
-    cat("     ++ Percent of lack of complete independence rebuilt  ++ ",
-        sum(pctotafc[show][-1]), "%", "\n")
-    cat("                                    selected pctoafc > ",
-        testvar, "%  total= ", sum(pctotafc[show & pctotafc >
-            testvar][-1]), "\n")
-    summ <- matrix(cbind(1:length(solution[[k]]$d), solution[[k]]$d,
-        solution[[k]]$ssX, pctota, pctotafc), ncol = 5)
+    cat("     ++ Percent of lack of ",wha, " rebuilt  ++ ",
+        ifelse(ismodel,sum(pctotafc[show]),sum(pctotafc[show][-1])), "%", "\n")
+    cat("                    selected pctoafc > ",
+        testvar, "%  total= ",  ifelse(ismodel,sum(pctotafc[show & pctotafc >
+            testvar]),sum(pctotafc[show & pctotafc >
+            testvar][-1])), "\n")
+    summ <- matrix(cbind(1:length(object[[k]]$d), object[[k]]$d,
+        object[[k]]$ssX, pctota, pctotafc), ncol = 5)
     summ <- summ[pctotafc > testvar & show, ]
     summ <- matrix(summ, ncol = 5)
-    summ[1, 5] <- NA
-    dimnames(summ) <- list(solution[[k]]$vsnam[pctotafc > testvar &
+    if(!ismodel)summ[1, 5] <- NA
+    dimnames(summ) <- list(object[[k]]$vsnam[pctotafc > testvar &
         show], c("-no-", "--Sing Val--", "--ssX--", "--Global Pct--",
         "--FCA--"))
     print(summ, digits = 5)
     cat("\n", "++++               ++++", "\n")
     if (!is.null(testvar) & !testvar == 0)
-        cat(" Shown are selected  over ", length(solution[[k]]$vsnam[nostar]) -
-            1, " PT  with pct AFC >", testvar, "% ", "\n")
-    else cat(" over ", length(solution[[k]]$vsnam), " PT (with*)",
+        cat(" Shown are selected  over ", length(object[[k]]$vsnam[show]) -
+            1, " PT  with pct FCA >", testvar, "% ", "\n")
+    else cat(" over ", length(object[[k]]$vsnam), " PT (with*)",
         "\n")
+  invisible(summ)
 }
 "summary.PTAk" <-
 function (object, testvar = 1, dontshow = "*",...)
 {
-    if (!is.null(dontshow))
-        if (dontshow == "*")
-            dontshow <- (!substr(object[[length(object)]]$vsnam,
-                1, 1) == "*")
-    RESUM(object, summary = TRUE, testvar = testvar, with = dontshow)
+           if (is.character(dontshow)){
+    		if(dontshow=="*") dontshow="[*]"
+        	  show <- !(grepl(dontshow, object[[length(object)]]$vsnam)) 
+        	  }
+   	 	if (is.logical(dontshow))show <-!dontshow 
+    RESUM(object, summary = TRUE, testvar = testvar, with = show)
 }
 "APSOLU3" <-
 function (X, solu, pt3 = NULL, nbPT2 = 1, smoothing = FALSE,
@@ -961,7 +981,7 @@ function (X, solu, pt3 = NULL, nbPT2 = 1, smoothing = FALSE,
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:3, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -1078,7 +1098,7 @@ function (X, solu, nbPT, nbPT2 = 1, smoothing = FALSE, smoo = list(NA),
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:diX, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -1175,6 +1195,9 @@ function (X, nbPT = 3, nbPT2 = 1, minpct = 0.01, smoothing = FALSE,
     addedcomment = "", chi2 = TRUE, E = NULL, ...)
 {
     ldx <- length(dim(X))
+    if (ldx <=2) {
+        stop(paste("--- X must be an array  of k > 2 entries! ---"))
+    }
     if (verbose) {
         cat("\n", "       ----------+++++++++++------------",
             "\n", ifelse(smoothing, paste("Penalised ", "\n"),
@@ -1204,7 +1227,45 @@ function (X, nbPT = 3, nbPT2 = 1, minpct = 0.01, smoothing = FALSE,
     solutions[[ldx]]$datanam <- substitute(X)
     solutions[[ldx]]$method <- match.call()
     solutions[[ldx]]$addedcomment <- addedcomment
-    class(solutions) <- c("FCAk", "PTAk")
+    class(solutions) <- c("FCAk","PTAk")
+    if(!is.null(E)) class(solutions) <- c("FCAk","PTAk","E=")
+    invisible(solutions)
+}
+"FCA2" <-
+function (X, nbdim =NULL, minpct = 0.01, smoothing = FALSE,
+    smoo = rep(list(function(u) ksmooth(1:length(u), u, kernel = "normal",
+        bandwidth = 3, x.points = (1:length(u)))$y), length(dim(X))),
+    verbose = getOption("verbose"), file = NULL, modesnam = NULL,
+    addedcomment = "", chi2 = FALSE, E = NULL, ...)
+{
+    ldx <- length(dim(X))
+    if (ldx !=2) {
+        stop(paste("--- X must be an array  of k = 2 entries! ---"))
+    }
+    if (verbose) {
+        cat("\n", "       ----------+++++++++++------------",
+            "\n", ifelse(smoothing, paste("Penalised ", "\n"),
+                ""), "       Correspondence Analysis on ", ldx,
+            " modes ", "\n", file = ifelse(is.null(file), "",
+                file), append = TRUE)
+        if (!is.null(modesnam))
+            cat("modes are ", modesnam, "\n", file = ifelse(is.null(file),
+                "", file), append = TRUE)
+        cat("       ----------+++++++++++------------", "\n",
+            fil = ifelse(is.null(file), "", file), append = TRUE)
+        cat("Data   = complete independence    + lack of independence ...",
+            "\n", file = ifelse(is.null(file), "", file), append = TRUE)
+        cat(" lack of independence = partial independence + lack of independence ... etc ...",
+            "\n", file = ifelse(is.null(file), "", file), append = TRUE)
+    }
+    Y <- FCAmet(X, chi2 = chi2, E = E)
+     solutions <- SVDgen(Y,smoothing = smoothing, nomb=nbdim, smoo = smoo)
+    
+    solutions[[ldx]]$datanam <- substitute(X)
+    solutions[[ldx]]$method <- match.call()
+    solutions[[ldx]]$addedcomment <- addedcomment
+    class(solutions) <- c("FCAk", "FCA2","PTAk")
+    if(!is.null(E)) class(solutions) <- c("FCAk","FCA2","PTAk","E=")
     invisible(solutions)
 }
 "PPMA" <-
@@ -1301,7 +1362,7 @@ function (X, nbPT = 2, nbPT2 = 1, smoothing = FALSE, smoo = list(function(u) ksm
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:3, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -1340,6 +1401,9 @@ function (X, nbPT = 2, nbPT2 = 1, smoothing = FALSE, smoo = list(function(u) ksm
     if (!is.array(X)) {
         stop(paste("--- X must be an array  ! ---"))
     }
+    if (length(dim(X)) <=2) {
+        stop(paste("--- X must be an array  of k > 2 entries! ---"))
+    }
     solutions <- NULL
     if (smoothing) {
         if (smoothing & !length(smoo) == 3) 
@@ -1456,7 +1520,7 @@ function (X, nbPT = 2, nbPT2 = 1, minpct = 0.1, smoothing = FALSE,
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:diX, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -1493,6 +1557,9 @@ function (X, nbPT = 2, nbPT2 = 1, minpct = 0.1, smoothing = FALSE,
         stop(paste("--- X must be an array  ! ---"))
     }
     kor <- length(dim(X))
+    if (kor <=2) {
+        stop(paste("--- X must be an array  of k > 2 entries! ---"))
+    }
     if (length(nbPT) < kor - 2) {
         nbPT <- rep(nbPT[1], kor - 2)
     }
@@ -1649,7 +1716,7 @@ function (X, test = 1e-12, PTnam = "vs111", Maxiter = 2000, verbose = getOption(
                 t12 <- array(t12, (dim(X$data))[laperm])
                 X$data <- aperm(t12, match(1:diX, laperm))
             }
-            else X$data <- X$data * X$met[[d]]
+            else X$data <- X$data * sqrt(X$met[[d]])
         }
         met <- X$met
         X <- X$data
@@ -1803,7 +1870,7 @@ function (X, test = 1e-12, PTnam = "vs111", Maxiter = 2000, verbose = getOption(
     return(sval)
 }
 "SVDgen" <-
-function (Y, D2 = 1, D1 = 1, smoothing = FALSE, nomb = min(dim(Y)),
+function (Y, D2 = 1, D1 = 1, smoothing = FALSE, nomb = NULL,
     smoo = list(function(u) ksmooth(1:length(u), u, kernel = "normal",
         bandwidth = 3, x.points = (1:length(u)))$y)){
     
@@ -1877,6 +1944,7 @@ function (Y, D2 = 1, D1 = 1, smoothing = FALSE, nomb = min(dim(Y)),
     solutions[[2]]$ssX <- rep(ssX, nomb)
     solutions[[2]]$vsnam <- paste("vs", 1:nomb, sep = "")
     solutions[[2]]$datanam <- datanam
+    solutions[[2]]$addedcomment <- ""
     solutions[[2]]$method <- match.call()
     if (smoothing)
         solutions[[2]]$smoocheck <- result$smoocheck
@@ -2079,18 +2147,16 @@ function (X, chi2 = FALSE, E = NULL,No0margins=TRUE)
             
           # evalCh.f(paste("X[[",names(X)[t],"=the0]]=amin",sep=""))
              # t th position X[,,,theo,,]=amin
-           evalCh.f(paste("X[",  pass.(",",(t-1)),"the0", pass.(",",(ord-t)), "]=rep(amin,length(theo))",sep=""))
+           evalCh.f(paste("X[",  pass.(",",(t-1)),"the0", pass.(",",(ord-t)), "]=rep(amin,length(the0))",sep=""))
             
           }
        }
        #X=array(as.vector(X),dim(X))
        #dimnames(X)=dnam 
-     }
-      
-      N <- sum(X)
+     } # X rebuilt if zeos margins
       
+      N <- sum(X)  
      metafc[[1]] <- apply(X, 1, sum)/N
-        
      Indep <-  metafc[[1]]
     for (t in 2:ord) {
         metafc[[t]] <- apply(X, t, sum)/N
@@ -2244,13 +2310,14 @@ function (A, B)
     }
 }
 "RiskJackplot" <-
-function (x, nbvs = 1:20, mod = NULL, max = NULL, rescaled = TRUE,
+function (x, nbvs = 1:20, mod = NULL, max = NULL, rescaled = TRUE, 
     ...)
 {     solution <- x
     qchoix <- nbvs
     ord <- length(solution)
     if (is.null(max))
         max <- length(solution[[ord]]$d)
+    if(max(nbvs) >= 0.8*max)warning("nbvs could be too high!")    
     if (is.null(mod))
         mod <- 1:length(solution)
     val <- solution[[ord]]$d[!substr(solution[[ord]]$vsnam, 1,
@@ -2301,7 +2368,7 @@ function (x, nbvs = 1:20, mod = NULL, max = NULL, rescaled = TRUE,
             lty = u, col = u, type = "b", ...)
         par(new = TRUE)
     }
-    legend(max(qchoix) - 1.5, max(RJack)/2, paste("Risk-mode",
+    legend(2, max(RJack)/2, paste("Risk-mode",
         mod), col = mod, lty = mod, bty = "n", cex = 0.7)
     invisible(par(new = FALSE))
 }
@@ -2338,12 +2405,103 @@ function (y, x = NULL, sigmak = NULL, sigmat = NULL, ker = list(function(u) retu
     }
     return(resul)
 }
+"COS2" <-function(solu,mod=1,solnbs=2:4){
+	# as (t(phi_s) D phi_s)i/(t(vec_phi) D_-phi vec_phi ) if normed to lambda_s otherwise has to be times lambda_s
+	# can be added on the upto for accumulated "rendering" 
+	
+    X <-eval(solu[[length(solu)]]$datanam)
+                   
+		if("FCAk" %in% class(solu)){
+				if("E" %in% names(solu[[length(solu)]]$method)){
+					leE=eval((solu[[length(solu)]]$method)$E)
+				 X <-FCAmet(X,E=leE)				
+				}
+				else{				
+						X <-FCAmet(X)
+						X$data <- X$data -1
+				} 
+		}					 
+	 # norm of the mod vv
+	
+	 if(is.list(X)) {
+	 	# XD1/2 except mod
+        nam <- dimnames(X$data)
+        diX <- length(dim(X$data))
+        for (d in 1:diX) {
+        	if (d==mod){X$met[[d]]=1}
+           
+            if (length(X$met[[d]]) > 1) {
+                if (length(X$met[[d]]) == dim(X$data)[d]^2) {
+                  tempp <- d
+                  t12 <- CONTRACTION(X$data, Powmat(X$met[[d]],
+                    1/2), Xwiz = d, zwiX = 1)
+                  d <- tempp
+                  lacola <- (1:diX)[-d]
+                  laperm <- c(lacola, d)
+                }
+                else {
+                  lacola <- (1:diX)[-d]
+                  laperm <- c(d, lacola)
+                  lacol <- (dim(X$data))[lacola]
+                  pt12 <- matrix(aperm(X$data, laperm), ncol = prod(lacol))
+                  t12 <- sqrt(X$met[[d]]) * pt12
+                }
+                t12 <- array(t12, (dim(X$data))[laperm])
+                X$data <- aperm(t12, match(1:diX, laperm))
+            }
+            else X$data <- X$data * sqrt(X$met[[d]])
+        }
+       # metAll <- X$met
+        X <- X$data
+        dimnames(X) <- nam
+    }
+    
+     diX <-length(dim(X))
+       lacola <- (1:diX)[-mod]
+           laperm <- c(mod, lacola)
+              lacol <- (dim(X))[lacola]
+       Xmod <- matrix(aperm(X, laperm), ncol = prod(lacol))	 	 
+	 
+	   
+	  cos2 <- (solu[[mod]]$v[solnbs,]**2)*rep(solu[[length(solu)]]$d[solnbs]**2,dim(solu[[mod]]$v)[2])
+	  vv <- apply(Xmod**2,1,sum)
+	cos2 <- t(cos2)/vv
+	
+	 colnames(cos2) <- paste(rep("cos2",length(solnbs)),paste(":",solnbs,":",sep=""),solu[[length(solu)]]$vsnam[solnbs],sep="_")
+	 rownames(cos2) <- solu[[mod]]$n
+	return(round(cos2*1000))
+}
+##############
+"CTR"  <-function(solu, mod=1,solnbs=1:4){
+	# as ()t(phi_s) D phi_s)i/lambda_s  if normed to lambda_s otherwise not divided by lambda
+                    
+	if(is.list(eval(solu[[length(solu)]]$datanam))) {
+	   	met <- eval(solu[[length(solu)]]$datanam)$met[[mod]]}
+	else{
+		if("FCAk" %in% class(solu)){
+			met <-FCAmet(eval(solu[[length(solu)]]$datanam))$met[[mod]]
+			}
+		else{
+	   		met <-1
+	   	}
+	 } 
+	if(is.vector(met)){
+		ctr <- t(solu[[mod]]$v[solnbs,]**2)*met 
+		}
+	else{
+		ctr <- t( (solu[[mod]]$v[solnbs,]%*%met)*solu[[mod]]$v[solnbs,]) 
+	}
+	colnames(ctr) <- paste(rep("ctr",length(solnbs)),paste(":",solnbs,":",sep=""),solu[[length(solu)]]$vsnam[solnbs],sep="_")
+	rownames(ctr) <- solu[[mod]]$n
+	return(round(ctr*1000))
+}
 "plot.PTAk" <-
 function (x, labels = TRUE, mod = 1, nb1 = 1, nb2 = NULL,
     coefi = list(NULL, NULL), xylab = TRUE, ppch = (1:length(solution)),
-    lengthlabels = 2, ylimit = NULL, scree = FALSE, ordered = TRUE,
-    nbvs = 40, RiskJack = NULL, method = "", ...)
+    lengthlabels = 2, scree = FALSE, ordered = TRUE,
+    nbvs = 40, RiskJack = NULL, method = "",ZoomInOut=NULL, Zlabels=NULL, Zcol=NULL,...)
 {      solution <- x
+	awaybor=1.04
 if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot function not available yet using Plot.PTAk!","\n")
     if (is.null(coefi[[1]]))
         coefi[[1]] <- rep(1, length(solution))
@@ -2354,8 +2512,7 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
     if (length(lengthlabels) == 1)
         lengthlabels <- rep(lengthlabels, length(solution))
     ord <- length(solution)
-    if (as.character(solution[[ord]]$method)[1] == "FCA" | method ==
-        "FCA") {
+    if ("FCAk" %in% class(x) && !("E=" %in% class(x)) ) {
         divv <- solution[[ord]]$ssX[1] - 1
         perclab <- "% FCA"
         if (length(nbvs) == 1)
@@ -2371,9 +2528,10 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
     for (r in 1:length(solution)) di <- c(di, length(solution[[r]]$v[1,
         ]))
     if (!scree) {
-        ylim <- ylimit
         xlab <- ""
         ylab <- ""
+        ylim <- NULL
+        xlim <- NULL
         if (is.null(nb2)) {
             xlim <- c(1, max(di[mod]) + 1)
             if (xylab)
@@ -2391,42 +2549,38 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
                   round(solution[[ord]]$pct[nb2], 2), "% ", round((100 *
                     (solution[[ord]]$d[nb2])^2)/divv, 2), perclab)
         }
-        for (u in mod) {
-            if (!is.null(nb2)) {
-                xy <- t(solution[[u]]$v[c(nb1, nb2), ]) %*% diag(c(coefi[[1]][u],
-                  coefi[[2]][u]))
-                xyn <- t(solution[[u]]$v[c(nb1, nb2), ]) %*%
-                  diag(c(coefi[[1]][u], coefi[[2]][u]))
-                ylim <- c(min(ylim, xyn), max(ylim, xyn))
-            }
-            else {
-                xy <- solution[[u]]$v[nb1, ] * coefi[[1]][u]
-                xyn <- solution[[u]]$v[nb1, ] * coefi[[1]][u]
-                ylim <- c(min(ylim, xyn), max(ylim, xyn))
-            }
-        }
         for (u in mod) {  
             if (!is.null(nb2)) {
-                xy <- t(solution[[u]]$v[c(nb1, nb2), ]) %*% diag(c(coefi[[1]][u],
-                  coefi[[2]][u]))
-                xyn <- t(solution[[u]]$v[c(nb1, nb2), ]) %*%
-                  diag(c(coefi[[1]][u], coefi[[2]][u]))
-                ylim <- c(min(ylim, xyn), max(ylim, xyn))
-                xlim <- ylim
+                xyn <- t(solution[[u]]$v[c(nb1, nb2), ]) %*% diag(c(coefi[[1]][u], coefi[[2]][u]))                 
                 xaxt <- "s"
+                ylim <- range(xyn[,2],ylim)
+                xlim <- range(xyn[,1],xlim)
+                
             }
             else {
-                xy <- solution[[u]]$v[nb1, ] * coefi[[1]][u]
                 xyn <- solution[[u]]$v[nb1, ] * coefi[[1]][u]
-                ylim <- c(min(ylim, xyn), max(ylim, xyn))
-                if (!"xaxt" %in% names(list(...)))
-                  xaxt <- "n"
+               if (!"xaxt" %in% names(list(...)))
+                  xaxt <- "n" 
+				ylim <- range(xyn,ylim)               
             }
+		}
+		ylim <- awaybor *ylim
+		if (!is.null(nb2))xlim <- awaybor *xlim
+        if(!is.null(ZoomInOut[[2]])) ylim <- ZoomInOut[[2]]
+          if(!is.null(ZoomInOut[[1]]) &&  !is.null(nb2)) xlim <-ZoomInOut[[1]]
+
+        for (u in mod) { 
+        	 if (!is.null(nb2)) {
+                 xy <- t(solution[[u]]$v[c(nb1, nb2), ]) %*% diag(c(coefi[[1]][u],coefi[[2]][u]))
+             }
+             else {
+                 xy <- solution[[u]]$v[nb1, ] * coefi[[1]][u]
+             }         	
             if (labels) {
                 if ("xlab" %in% names(list(...))) {
                   if ("ylab" %in% names(list(...)))
                     plot(xy, xlim = xlim, ylim = ylim, pch = ppch[u],
-                      xaxt = xaxt, ...)
+                      xaxt = xaxt,...)
                   else plot(xy, xlim = xlim, ylim = ylim, ylab = ylab,
                     pch = ppch[u], xaxt = xaxt, ...)
                   if (is.null(nb2))
@@ -2440,26 +2594,40 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
                     pch = ppch[u], xaxt = xaxt, xlab = xlab,
                     ...)
                 }
-                if (!is.null(solution[[u]]$n)) {
-                  if (is.factor(solution[[u]]$n)) {
+              if(!is.null(Zlabels[[u]])){
+              	ZlabUN <- Zlabels[[u]] 
+              	       }
+              else {
+              	ZlabUN <- solution[[u]]$n
+              }  
+                if (!is.null(ZlabUN)) {
+                	ZcolUN= u
+              	if(!is.null(Zcol[[u]]))ZcolUN=Zcol[[u]]
+
+                  if (is.factor(ZlabUN)) {
                     if ("cex" %in% names(list(...)))
                       cex <- list(...)$cex
                     else cex <- par("cex")
-                    text(xy, labels = substr(levels(solution[[u]]$n)[as.numeric(solution[[u]]$n)],
-                      1, lengthlabels[u]), col = as.numeric(solution[[u]]$n),
+                    text(xy, labels = substr(levels(ZlabUN),
+                      1, lengthlabels[u]), col = ZcolUN,
                       pos = 4,...)
                     if (is.null(nb2)) {
                       par(new = TRUE)
-                      plot(xy ~ solution[[u]]$n, xlab = "", ylab = "",
+                      plot(xy ~ ZlabUN, xlab = "", ylab = "",
                         ylim = ylim, cex = cex)
                       par(new = FALSE)
                     }
                   }
-                  else text(xy, labels = substr(solution[[u]]$n,
-                    1, lengthlabels[u]), col = u, pos = 4,...)
+                  else { 
+                  	ZcolUN= u
+              	if(!is.null(Zcol[[u]]))ZcolUN=Zcol[[u]]
+                  	text(xy, labels = substr(ZlabUN,
+                    1, lengthlabels[u]), col = ZcolUN, pos = 4,...)
+                    }
                 }
-            }
-            else if ("xlab" %in% names(list(...))) {
+            } 
+            else 
+            if ("xlab" %in% names(list(...))) {
                 if ("ylab" %in% names(list(...)))
                   plot(xy, xlim = xlim, ylim = ylim, pch = ppch[u],
                     col = u, xaxt = xaxt, ...)
@@ -2476,8 +2644,10 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
                   pch = ppch[u], col = u, xaxt = xaxt, xlab = xlab,
                   ...)
             }
+            if(!is.null(nb2)) {
             abline(h = 0, col = "green", lty = 2)
             abline(v = 0, col = "green", lty = 2)
+            }
             par(new = TRUE)
         }
         invisible(par(new = FALSE))
@@ -2488,13 +2658,14 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
                 ld <- length(solution[[ord]]$d[!substr(solution[[ord]]$vsnam,
                   1, 1) == "*"])
                 if (length(nbvs) == 1) {
-                  nbvs <- min(max(5, nbvs), ld)
+                  nbvs <- min(max(3, nbvs), ld)
                   nbvs <- 1:nbvs
                 }
                 scre <- 100 * ((solution[[ord]]$d[!substr(solution[[ord]]$vsnam,
-                  1, 1) == "*"])^2)/divv
-                scre <- (sort(scre[nbvs]))
+                  1, 1) == "*"])^2)/divv 
+                scre <- (sort(scre[nbvs]))            
                 scre <- scre[length(scre):1]
+                if (!is.null(RiskJack)) scre <- scre[1:min(max(RiskJack,2),max(nbvs-2))]
                 nbvs <- nbvs[1:length(scre)]
                 plot(nbvs, scre, xlab = "Ordered ", ylab = "Squared Singular Values (%)",
                   xaxt = "n", ...)
@@ -2507,11 +2678,12 @@ if(class(solution)[1]=="PCAn" | class(solution)[1]=="CANDPARA" )cat("\n","Plot f
                   format = "fg"), col.axis = 3)
                 par(new = TRUE)
                 if (!is.null(RiskJack))
-                  RiskJackplot(solution, nbvs = nbvs, mod = mod,
-                    max = min(RiskJack + length(nbvs), ld), rescaled = TRUE,
+                  RiskJackplot(solution, nbvs = nbvs, mod = NULL,
+                    max = NULL, rescaled = TRUE,
                     axes = FALSE, ann = FALSE, pch = "r")
                 par(new = FALSE)
-            }
+            } 
+            
             if (ordered == FALSE) {
                 ld <- length(solution[[ord]]$d[!substr(solution[[ord]]$vsnam,
                   1, 1) == "*"])
